@@ -1,46 +1,38 @@
 use crate::{
   activities::{
-    community::announce::AnnouncableActivities,
-    deletion::{
-      receive_delete_action,
-      verify_delete_activity,
-      DeletableObjects,
-      WebsocketMessages,
-    },
+    community::{announce::GetCommunity, send_activity_in_community},
+    deletion::{receive_delete_action, verify_delete_activity, DeletableObjects},
     generate_activity_id,
     verify_activity,
+    verify_is_public,
   },
-  activity_queue::send_to_community_new,
-  extensions::context::lemmy_context,
-  fetcher::object_id::ObjectId,
-  ActorType,
-};
-use activitystreams::{
-  activity::kind::DeleteType,
-  base::AnyBase,
-  primitives::OneOrMany,
-  unparsed::Unparsed,
+  activity_lists::AnnouncableActivities,
+  objects::{community::ApubCommunity, person::ApubPerson},
+  protocol::activities::deletion::delete::Delete,
 };
+use activitystreams_kinds::{activity::DeleteType, public};
 use anyhow::anyhow;
 use lemmy_api_common::blocking;
-use lemmy_apub_lib::{values::PublicUrl, ActivityFields, ActivityHandler};
-use lemmy_db_queries::{
-  source::{comment::Comment_, community::Community_, post::Post_},
-  Crud,
+use lemmy_apub_lib::{
+  data::Data,
+  object_id::ObjectId,
+  traits::{ActivityHandler, ActorType},
 };
-use lemmy_db_schema::source::{
-  comment::Comment,
-  community::Community,
-  moderator::{
-    ModRemoveComment,
-    ModRemoveCommentForm,
-    ModRemoveCommunity,
-    ModRemoveCommunityForm,
-    ModRemovePost,
-    ModRemovePostForm,
+use lemmy_db_schema::{
+  source::{
+    comment::Comment,
+    community::Community,
+    moderator::{
+      ModRemoveComment,
+      ModRemoveCommentForm,
+      ModRemoveCommunity,
+      ModRemoveCommunityForm,
+      ModRemovePost,
+      ModRemovePostForm,
+    },
+    post::Post,
   },
-  person::Person,
-  post::Post,
+  traits::Crud,
 };
 use lemmy_utils::LemmyError;
 use lemmy_websocket::{
@@ -48,50 +40,23 @@ use lemmy_websocket::{
   LemmyContext,
   UserOperationCrud,
 };
-use serde::{Deserialize, Serialize};
-use serde_with::skip_serializing_none;
 use url::Url;
 
-/// This is very confusing, because there are four distinct cases to handle:
-/// - user deletes their post
-/// - user deletes their comment
-/// - remote community mod deletes local community
-/// - remote community deletes itself (triggered by a mod)
-///
-/// TODO: we should probably change how community deletions work to simplify this. Probably by
-/// wrapping it in an announce just like other activities, instead of having the community send it.
-#[skip_serializing_none]
-#[derive(Clone, Debug, Deserialize, Serialize, ActivityFields)]
-#[serde(rename_all = "camelCase")]
-pub struct Delete {
-  actor: ObjectId<Person>,
-  to: [PublicUrl; 1],
-  pub(in crate::activities::deletion) object: Url,
-  pub(in crate::activities::deletion) cc: [ObjectId<Community>; 1],
-  #[serde(rename = "type")]
-  kind: DeleteType,
-  /// If summary is present, this is a mod action (Remove in Lemmy terms). Otherwise, its a user
-  /// deleting their own content.
-  pub(in crate::activities::deletion) summary: Option<String>,
-  id: Url,
-  #[serde(rename = "@context")]
-  context: OneOrMany<AnyBase>,
-  #[serde(flatten)]
-  unparsed: Unparsed,
-}
-
 #[async_trait::async_trait(?Send)]
 impl ActivityHandler for Delete {
+  type DataType = LemmyContext;
   async fn verify(
     &self,
-    context: &LemmyContext,
+    context: &Data<LemmyContext>,
     request_counter: &mut i32,
   ) -> Result<(), LemmyError> {
-    verify_activity(self, &context.settings())?;
+    verify_is_public(&self.to, &[])?;
+    verify_activity(&self.id, self.actor.inner(), &context.settings())?;
+    let community = self.get_community(context, request_counter).await?;
     verify_delete_activity(
-      &self.object,
-      self,
-      &self.cc[0],
+      &self.object.id,
+      &self.actor,
+      &community,
       self.summary.is_some(),
       context,
       request_counter,
@@ -102,7 +67,7 @@ impl ActivityHandler for Delete {
 
   async fn receive(
     self,
-    context: &LemmyContext,
+    context: &Data<LemmyContext>,
     request_counter: &mut i32,
   ) -> Result<(), LemmyError> {
     if let Some(reason) = self.summary {
@@ -113,65 +78,57 @@ impl ActivityHandler for Delete {
       } else {
         Some(reason)
       };
-      receive_remove_action(&self.actor, &self.object, reason, context, request_counter).await
-    } else {
-      receive_delete_action(
-        &self.object,
+      receive_remove_action(
         &self.actor,
-        WebsocketMessages {
-          community: UserOperationCrud::DeleteCommunity,
-          post: UserOperationCrud::DeletePost,
-          comment: UserOperationCrud::DeleteComment,
-        },
-        true,
+        &self.object.id,
+        reason,
         context,
         request_counter,
       )
       .await
+    } else {
+      receive_delete_action(&self.object.id, &self.actor, true, context, request_counter).await
     }
   }
 }
 
 impl Delete {
   pub(in crate::activities::deletion) fn new(
-    actor: &Person,
-    community: &Community,
-    object_id: Url,
+    actor: &ApubPerson,
+    object: DeletableObjects,
     summary: Option<String>,
     context: &LemmyContext,
   ) -> Result<Delete, LemmyError> {
     Ok(Delete {
       actor: ObjectId::new(actor.actor_id()),
-      to: [PublicUrl::Public],
-      object: object_id,
-      cc: [ObjectId::new(community.actor_id())],
+      to: vec![public()],
+      object: object.to_tombstone()?,
       kind: DeleteType::Delete,
       summary,
       id: generate_activity_id(
         DeleteType::Delete,
         &context.settings().get_protocol_and_hostname(),
       )?,
-      context: lemmy_context(),
       unparsed: Default::default(),
     })
   }
   pub(in crate::activities::deletion) async fn send(
-    actor: &Person,
-    community: &Community,
-    object_id: Url,
+    actor: &ApubPerson,
+    community: &ApubCommunity,
+    object: DeletableObjects,
     summary: Option<String>,
     context: &LemmyContext,
   ) -> Result<(), LemmyError> {
-    let delete = Delete::new(actor, community, object_id, summary, context)?;
+    let delete = Delete::new(actor, object, summary, context)?;
     let delete_id = delete.id.clone();
 
     let activity = AnnouncableActivities::Delete(delete);
-    send_to_community_new(activity, &delete_id, actor, community, vec![], context).await
+    send_activity_in_community(activity, &delete_id, actor, community, vec![], context).await
   }
 }
 
 pub(in crate::activities) async fn receive_remove_action(
-  actor: &ObjectId<Person>,
+  actor: &ObjectId<ApubPerson>,
   object: &Url,
   reason: Option<String>,
   context: &LemmyContext,
@@ -241,3 +198,26 @@ pub(in crate::activities) async fn receive_remove_action(
   }
   Ok(())
 }
+
+#[async_trait::async_trait(?Send)]
+impl GetCommunity for Delete {
+  async fn get_community(
+    &self,
+    context: &LemmyContext,
+    _request_counter: &mut i32,
+  ) -> Result<ApubCommunity, LemmyError> {
+    let community_id = match DeletableObjects::read_from_db(&self.object.id, context).await? {
+      DeletableObjects::Community(c) => c.id,
+      DeletableObjects::Comment(c) => {
+        let post = blocking(context.pool(), move |conn| Post::read(conn, c.post_id)).await??;
+        post.community_id
+      }
+      DeletableObjects::Post(p) => p.community_id,
+    };
+    let community = blocking(context.pool(), move |conn| {
+      Community::read(conn, community_id)
+    })
+    .await??;
+    Ok(community.into())
+  }
+}
